@@ -162,19 +162,15 @@ class BluexpressHandler:
             }
             return [{}, {}, ...]
         """
-        parse = xmltodict.parse(raw_data.get('tracking_data'))
-        json_data = json.loads(json.dumps(parse))
-        list_keys = list(json_data['soap:Envelope']['soap:Body'].keys())
-        documento = json_data['soap:Envelope']['soap:Body'][list_keys[0]]['listaDocumento']['documento']
-        pinchazos = documento['listaPinchazosNacionales']['pinchazo']
+        document = self._extract_document(raw_data.get('tracking_data'))
 
         events = [
             {
-                'city': pinchazo['nombrePosta'],
-                'state': pinchazo['codigoPosta'],
-                'description': f"{pinchazo['codigoTipo']} - {pinchazo['nombreTipo']}",
-                'date': pinchazo['fecha'],
-            } for pinchazo in pinchazos
+                'city': prick.get('nombrePosta'),
+                'state': prick.get('codigoPosta'),
+                'description': f"{prick.get('codigoTipo')} - {prick.get('nombreTipo')}",
+                'date': prick.get('fecha'),
+            } for prick in document['listaPinchazosNacionales']['pinchazo']
         ]
 
         return events
@@ -190,15 +186,38 @@ class BluexpressHandler:
 
             response: ('Entregado', True)
         """
-        parse = xmltodict.parse(raw_data.get('tracking_data'))
-        json_data = json.loads(json.dumps(parse))
-        list_keys = list(json_data['soap:Envelope']['soap:Body'].keys())
-        documento = json_data['soap:Envelope']['soap:Body'][list_keys[0]]['listaDocumento']['documento']
-        macroestados = documento['listaMacroestados']['macroestado']
-        status = macroestados[-1]['nombre']
+        document = self._extract_document(raw_data.get('tracking_data'))
+        macroestates = document['listaMacroestados']['macroestado']
+        status = macroestates[-1].get('nombre')
         is_delivered = False
 
+        if not status:
+            status = macroestates[-2].get('nombre')
+
         if status == 'Entregado':
             is_delivered = True
 
         return status, is_delivered
+
+    def _extract_document(self, tracking_data):
+        """
+            Extract document from xml string.
+        """
+
+        parse = xmltodict.parse(tracking_data)
+        json_data = json.loads(json.dumps(parse))
+        envelope = self._extract_full_key(json_data, 'Envelope')
+        envelope_data = json_data[envelope]
+        body = self._extract_full_key(envelope_data, 'Body')
+        body_data = envelope_data[body]
+        response = self._extract_full_key(body_data, 'responseObtenerDocumento')
+        response_data = body_data[response]
+        return response_data['listaDocumento']['documento']
+
+    def _extract_full_key(self, json_data, search_key):
+        """
+            Extract full key from tag.
+        """
+        for data in json_data:
+            if search_key in data.split(':'):
+                return data
